@@ -1,5 +1,6 @@
 import asyncio
 from unittest import IsolatedAsyncioTestCase
+from typing import Any
 
 from ourcompose.framework import Composition, Piece, Dependency, Competition
 
@@ -976,6 +977,118 @@ class TestFramework(IsolatedAsyncioTestCase):
         except Exception as e:
             self.fail(e)
 
+    async def test_competition_of_compositions_when_one_fail_one_sleep_one_succeeds(self):
+
+        async def fn0(d: dict) -> list:
+            await asyncio.sleep(1)
+            return list(d.keys())
+
+        async def fn1(d: dict) -> list:
+            raise Exception("Did not finish")
+
+        async def fn2(d: dict) -> list:
+            return list(d.values())
+
+        competition = Competition(
+            id="my-competition",
+            pieces=[
+                Composition(
+                    id="my-comp-0",
+                    pieces=[
+                        Piece(id="contestent-0", fn=fn0),
+                    ],
+                ),
+                Composition(
+                    id="my-comp-0",
+                    pieces=[
+                        Piece(id="contestent-1", fn=fn1),
+                    ],
+                ),
+                Composition(
+                    id="my-comp-0",
+                    pieces=[
+                        Piece(id="contestent-2", fn=fn2),
+                    ],
+                ),
+            ],
+        )
+
+        try:
+            result = await competition.perform({"d": {"a": 1, "b": 2, "c": 0}})
+            self.assertEqual(result, [1,2,0])
+        except Exception as e:
+            self.fail(e)
+
+    async def test_competition_of_competitions_of_compositions_when_one_fail_one_sleep_one_succeeds(self):
+
+        async def fn0(d: dict) -> list:
+            await asyncio.sleep(1)
+            return list(d.keys())
+
+        async def fn1(d: dict) -> list:
+            raise Exception("Did not finish")
+
+        async def fn2(d: dict) -> list:
+            return list(d.values())
+
+        competition = Competition(
+            id="my-competition",
+            pieces=[
+                Competition(
+                    id="ct-0", 
+                    pieces=[
+                        Composition(
+                            id="my-comp-0",
+                            pieces=[
+                                Piece(id="contestent-0", fn=fn0),
+                            ],
+                        ),
+                        Composition(
+                            id="my-comp-0",
+                            pieces=[
+                                Piece(id="contestent-1", fn=fn1),
+                            ],
+                        ),
+                        Composition(
+                            id="my-comp-0",
+                            pieces=[
+                                Piece(id="contestent-2", fn=fn2),
+                            ],
+                        ),
+                    ],
+                ),
+                Competition(
+                    id="ct-1", 
+                    pieces=[
+                        Composition(
+                            id="my-comp-0",
+                            pieces=[
+                                Piece(id="contestent-0", fn=fn0),
+                            ],
+                        ),
+                        Composition(
+                            id="my-comp-0",
+                            pieces=[
+                                Piece(id="contestent-1", fn=fn1),
+                            ],
+                        ),
+                        Composition(
+                            id="my-comp-0",
+                            pieces=[
+                                Piece(id="contestent-2", fn=fn2),
+                            ],
+                        ),
+                    ],
+                ),
+            ],
+        )
+
+        try:
+            result = await competition.perform({"d": {"a": 1, "b": 2, "c": 0}})
+            self.assertEqual(result, [1,2,0])
+        except Exception as e:
+            self.fail(e)
+
     async def test_competition_of_compositions_and_pieces_will_pass(self):
 
         async def fn00(d: dict) -> list:
@@ -1085,7 +1198,7 @@ class TestFramework(IsolatedAsyncioTestCase):
     async def test_compete_with_competitions_reverse_should_pass(self):
 
         async def fn0(d: list) -> list:
-            await asyncio.sleep(0.5)
+            await asyncio.sleep(1)
             return [x-1 for x in d]
 
         async def fn1(d: list) -> list:
@@ -1256,4 +1369,33 @@ class TestFramework(IsolatedAsyncioTestCase):
             self.assertEqual(results, [comp.performer_results()['fn1'] for _, comp in inputs])
         except Exception as e:
             self.fail(e)
+
+    async def test_when_all_competition_pieces_fail_exceptions_should_be_returned(self):
+
+        async def fn0(d: Any) -> list:
+            raise Exception("Exception 1!")
+
+        async def fn1(d: dict) -> list:
+            raise Exception("Exception 2!")
+
+        competition = Competition(
+            id="comp",
+            pieces=[
+                Piece(
+                    id="fn0",
+                    fn=fn0,
+                ),
+                Piece(
+                    id="fn1",
+                    fn=fn1,
+                ),
+            ],
+        )
+
+        try:
+            _ = await competition.perform({'d': {"a": 1}})
+            self.fail("Did not raise an exception")
+        except:
+            pass
+        
         
\ No newline at end of file
